@@ -9,26 +9,23 @@ BACKEND_TO_SERVICE_MAPPING = {"GCPPubSubBackend": Service}
 
 
 class Child:
-    """A class representing an Octue child service that can be asked questions. It is a convenience wrapper for
-    `Service` that makes question asking more intuitive and allows easier selection of backends.
+    """A class representing an Octue child service that can be asked questions. This is a convenience wrapper for
+    `Service` that makes asking questions more intuitive and allows easier selection of backends.
 
     :param str id: the ID of the child
     :param dict backend: must include the key "name" with a value of the name of the type of backend e.g. "GCPPubSubBackend" and key-value pairs for any other parameters the chosen backend expects
-    :param str|None internal_service_name: the name to give to the internal service used to ask questions to the child
+    :param str internal_service_name: the name to give to the internal service used to ask questions to the child
     :return None:
     """
 
-    def __init__(self, id, backend, internal_service_name=None):
+    def __init__(self, id, backend, internal_service_name="local/local:local"):
         self.id = id
 
         backend = copy.deepcopy(backend)
         backend_type_name = backend.pop("name")
         backend = service_backends.get_backend(backend_type_name)(**backend)
 
-        self._service = BACKEND_TO_SERVICE_MAPPING[backend_type_name](
-            name=internal_service_name or f"{self.id}-parent",
-            backend=backend,
-        )
+        self._service = BACKEND_TO_SERVICE_MAPPING[backend_type_name](name=internal_service_name, backend=backend)
 
     def __repr__(self):
         """Represent the child as a string.
@@ -37,6 +34,15 @@ class Child:
         """
         return f"<{type(self).__name__}({self.id!r})>"
 
+    @property
+    def received_messages(self):
+        """Get the messages received from the child if it has been asked a question. If it hasn't, `None` is returned.
+        If an empty list is returned, no messages have been received.
+
+        :return list(dict)|None:
+        """
+        return self._service.received_messages
+
     def ask(
         self,
         input_values=None,
@@ -45,7 +51,7 @@ class Child:
         subscribe_to_logs=True,
         allow_local_files=False,
         handle_monitor_message=None,
-        record_messages_to=None,
+        record_messages=True,
         allow_save_diagnostics_data_on_crash=True,
         question_uuid=None,
         timeout=86400,
@@ -60,7 +66,7 @@ class Child:
         :param bool subscribe_to_logs: if `True`, subscribe to logs from the child and handle them with the local log handlers
         :param bool allow_local_files: if `True`, allow the input manifest to contain references to local files - this should only be set to `True` if the child will have access to these local files
         :param callable|None handle_monitor_message: a function to handle monitor messages (e.g. send them to an endpoint for plotting or displaying) - this function should take a single JSON-compatible python primitive as an argument (note that this could be an array or object)
-        :param str|None record_messages_to: if given a path to a JSON file, messages received in response to the question are saved to it
+        :param bool record_messages: if `True`, record messages received from the child in the `received_messages` property
         :param bool allow_save_diagnostics_data_on_crash: if `True`, allow the input values and manifest (and its datasets) to be saved by the child if it fails while processing them
         :param str|None question_uuid: the UUID to use for the question if a specific one is needed; a UUID is generated if not
         :param float timeout: time in seconds to wait for an answer before raising a timeout error
@@ -82,7 +88,7 @@ class Child:
         return self._service.wait_for_answer(
             subscription=subscription,
             handle_monitor_message=handle_monitor_message,
-            record_messages_to=record_messages_to,
+            record_messages=record_messages,
             service_name=self.id,
             timeout=timeout,
         )
