@@ -1,10 +1,9 @@
-import importlib
+import copy
 import json
 import logging
 import logging.handlers
 import os
 import re
-import sys
 
 import google.api_core.exceptions
 from google import auth
@@ -13,10 +12,11 @@ from jsonschema import ValidationError, validate as jsonschema_validate
 
 import twined.exceptions
 from octue import exceptions
+from octue.app_loading import AppFrom
 from octue.cloud import storage
 from octue.cloud.storage import GoogleCloudStorageClient
-from octue.log_handlers import apply_log_handler, create_octue_formatter, get_log_record_attributes_for_environment
-from octue.resources import Child
+from octue.log_handlers import AnalysisLogHandlerSwitcher
+from octue.resources import Child, Dataset
 from octue.resources.analysis import CLASS_MAP, Analysis
 from octue.utils import gen_uuid
 from octue.utils.encoders import OctueJSONEncoder
@@ -66,7 +66,14 @@ class Runner:
 
         self.output_location = output_location
 
+        self.crash_diagnostics = {
+            "questions": [],
+            "configuration_values": copy.deepcopy(configuration_values),
+            "configuration_manifest": copy.deepcopy(configuration_manifest),
+        }
+
         self.crash_diagnostics_cloud_path = crash_diagnostics_cloud_path
+        self._storage_client = None
 
         # Ensure the twine is present and instantiate it.
         if isinstance(twine, Twine):
@@ -87,6 +94,13 @@ class Runner:
         self.service_id = service_id
         self._project_name = project_name
 
+    def __repr__(self):
+        """Represent the runner as a string.
+
+        :return str: the runner represented as a string.
+        """
+        return f"<{type(self).__name__}({self.service_id!r})>"
+
     def run(
         self,
         analysis_id=None,
@@ -97,7 +111,6 @@ class Runner:
         analysis_log_handler=None,
         handle_monitor_message=None,
         allow_save_diagnostics_data_on_crash=True,
-        sent_messages=None,
     ):
         """Run an analysis.
 
@@ -109,9 +122,11 @@ class Runner:
         :param logging.Handler|None analysis_log_handler: the logging.Handler instance which will be used to handle logs for this analysis run. Handlers can be created as per the logging cookbook https://docs.python.org/3/howto/logging-cookbook.html but should use the format defined above in LOG_FORMAT.
         :param callable|None handle_monitor_message: a function that sends monitor messages to the parent that requested the analysis
         :param bool allow_save_diagnostics_data_on_crash: if `True`, allow the input values and manifest (and its datasets) to be saved if the analysis fails
-        :param list|None sent_messages: the list of messages sent by the service running this runner (this should update in real time) to save if crash diagnostics are enabled
         :return octue.resources.analysis.Analysis:
         """
+        self.crash_diagnostics["input_values"] = copy.deepcopy(input_values)
+        self.crash_diagnostics["input_manifest"] = copy.deepcopy(input_manifest)
+
         if hasattr(self.twine, "credentials"):
             self._populate_environment_with_google_cloud_secrets()
             credentials = self.twine.credentials
@@ -141,14 +156,7 @@ class Runner:
             )
 
         if inputs["children"] is not None:
-            inputs["children"] = {
-                child["key"]: Child(
-                    id=child["id"],
-                    backend=child["backend"],
-                    internal_service_name=self.service_id,
-                )
-                for child in inputs["children"]
-            }
+            inputs["children"] = self._instantiate_children(inputs["children"])
 
         outputs_and_monitors = self.twine.prepare("monitor_message", "output_values", "output_manifest", cls=CLASS_MAP)
 
@@ -179,53 +187,37 @@ class Runner:
             )
 
             try:
-                # App as a class that takes "analysis" as a constructor argument and contains a method named "run" that
-                # takes no arguments.
-                if isinstance(self.app_source, type):
-                    self.app_source(analysis).run()
-
-                # App as a module containing a function named "run" that takes "analysis" as an argument.
-                elif hasattr(self.app_source, "run"):
-                    self.app_source.run(analysis)
-
-                # App as a string path to a module containing a class named "App" or a function named "run". The same
-                # other specifications apply as described above.
-                elif isinstance(self.app_source, str):
-
-                    with AppFrom(self.app_source) as app:
-                        if hasattr(app.app_module, "App"):
-                            app.app_module.App(analysis).run()
-                        else:
-                            app.app_module.run(analysis)
-
-                # App as a function that takes "analysis" as an argument.
-                else:
-                    self.app_source(analysis)
+                self._load_and_run_app(analysis)
 
             except ModuleNotFoundError as e:
                 raise ModuleNotFoundError(f"{e.msg} in {os.path.abspath(self.app_source)!r}.")
 
             except Exception as analysis_error:
+                logger.error(str(analysis_error))
+
                 if allow_save_diagnostics_data_on_crash:
                     if not self.crash_diagnostics_cloud_path:
                         logger.warning(
-                            "Cannot save crash diagnostics as the child doesn't have a crash diagnostics cloud path "
-                            "set."
+                            "Cannot save crash diagnostics as the child doesn't have the "
+                            "`crash_diagnostics_cloud_path` field set in its service configuration (`octue.yaml` file)."
                         )
 
                     else:
                         logger.warning("Saving crash diagnostics to %r.", self.crash_diagnostics_cloud_path)
 
                         try:
-                            self._save_crash_diagnostics_data(analysis, sent_messages)
+                            self._save_crash_diagnostics_data(analysis)
                             logger.warning("Crash diagnostics saved.")
                         except Exception as crash_diagnostics_save_error:
                             logger.error("Failed to save crash diagnostics.")
                             raise crash_diagnostics_save_error
 
-                logger.error(str(analysis_error))
                 raise analysis_error
 
+            finally:
+                if analysis._periodic_monitor_message_sender:
+                    analysis._periodic_monitor_message_sender.cancel()
+
             if not analysis.finalised:
                 analysis.finalise()
 
@@ -252,7 +244,9 @@ class Runner:
 
         for credential in missing_credentials:
             secret_path = secrets_client.secret_version_path(
-                project=project_name, secret=credential["name"], secret_version="latest"
+                project=project_name,
+                secret=credential["name"],
+                secret_version="latest",
             )
 
             try:
@@ -293,190 +287,148 @@ class Runner:
 
                     raise twined.exceptions.invalid_contents_map[manifest_kind](message)
 
-    def _save_crash_diagnostics_data(self, analysis, sent_messages):
-        """Save the values, manifests, and datasets for the analysis configuration and inputs to the crash diagnostics
-        cloud path.
+    def _instantiate_children(self, serialised_children):
+        """Instantiate children from their serialised form (e.g. as given in the app configuration) so they are ready
+        to be asked questions. For crash diagnostics, each child's `ask` method is wrapped so the runner can record the
+        questions asked by the app, the responses received to each question, and the order the questions are asked in.
 
-        :param octue.resources.analysis.Analysis analysis:
-        :param list|None sent_messages: the list of messages sent by the service running this runner (this should update in real time) to save if crash diagnostics are enabled
+        :param list(dict) serialised_children: serialised children from e.g. the app configuration file
+        :return dict: a mapping of child keys to `octue.resources.child.Child` instances
+        """
+        children = {}
+
+        for uninstantiated_child in serialised_children:
+            child = Child(
+                id=uninstantiated_child["id"],
+                backend=uninstantiated_child["backend"],
+                internal_service_name=self.service_id,
+            )
+
+            child.ask = self._add_child_question_and_response_recording(child, uninstantiated_child["key"])
+            children[uninstantiated_child["key"]] = child
+
+        return children
+
+    def _add_child_question_and_response_recording(self, child, key):
+        """Add question and response recording to the `ask` method of the given child. This allows the runner to record
+        the questions asked by the app, the responses received to each question, and the order the questions are asked
+        in for crash diagnostics.
+
+        :param octue.resources.child.Child child: the child to add question and response recording to
+        :param str key: the key used to identify the child within the service
+        :return callable: the wrapped `Child.ask` method
+        """
+        # Copy the `ask` method to avoid an infinite recursion.
+        original_ask_method = copy.copy(child.ask)
+
+        def wrapper(*args, **kwargs):
+            # Convert args to kwargs so all inputs to the `ask` method can be recorded whether they're provided
+            # positionally or as keyword arguments.
+            kwargs.update(dict(zip(original_ask_method.__func__.__code__.co_varnames, args)))
+
+            try:
+                return original_ask_method(**kwargs)
+            finally:
+                self.crash_diagnostics["questions"].append(
+                    {"id": child.id, "key": key, **kwargs, "messages": child.received_messages}
+                )
+
+        return wrapper
+
+    def _load_and_run_app(self, analysis):
+        """Load and run the app on the given analysis object.
+
+        :param octue.resources.analysis.Analysis analysis: the analysis object containing the configuration and inputs to run the app on
         :return None:
         """
-        storage_client = GoogleCloudStorageClient()
+        # App as a class that takes "analysis" as a constructor argument and contains a method named "run" that
+        # takes no arguments.
+        if isinstance(self.app_source, type):
+            self.app_source(analysis).run()
+            return
+
+        # App as a module containing a function named "run" that takes "analysis" as an argument.
+        if hasattr(self.app_source, "run"):
+            self.app_source.run(analysis)
+            return
+
+        # App as a string path to a module containing a class named "App" or a function named "run". The same
+        # other specifications apply as described above.
+        if isinstance(self.app_source, str):
+            with AppFrom(self.app_source) as app:
+                if hasattr(app.app_module, "App"):
+                    app.app_module.App(analysis).run()
+                else:
+                    app.app_module.run(analysis)
+            return
+
+        # App as a function that takes "analysis" as an argument.
+        self.app_source(analysis)
+
+    def _save_crash_diagnostics_data(self, analysis):
+        """Save the following data to the crash diagnostics cloud path:
+        - Configuration values
+        - Configuration manifest and datasets
+        - Input values
+        - Input manifest and datasets
+        - Questions asked to any children during the analysis and any responses received
+
+        :param octue.resources.analysis.Analysis analysis:
+        :return None:
+        """
+        self._storage_client = GoogleCloudStorageClient()
         question_diagnostics_path = storage.path.join(self.crash_diagnostics_cloud_path, analysis.id)
 
         for data_type in ("configuration", "input"):
-
-            # Upload the configuration and input values.
             values_type = f"{data_type}_values"
-
-            if getattr(analysis, values_type):
-                storage_client.upload_from_string(
-                    json.dumps(getattr(analysis, values_type), cls=OctueJSONEncoder),
-                    cloud_path=storage.path.join(question_diagnostics_path, f"{values_type}.json"),
-                )
-
-            # Upload the configuration and input manifests.
             manifest_type = f"{data_type}_manifest"
 
-            if getattr(analysis, manifest_type):
-                manifest = getattr(analysis, manifest_type)
+            if self.crash_diagnostics[values_type] is not None:
+                self._upload_values(values_type, question_diagnostics_path)
 
-                for name, dataset in manifest.datasets.items():
-                    dataset.upload(storage.path.join(question_diagnostics_path, f"{manifest_type}_datasets", name))
+            if self.crash_diagnostics[manifest_type] is not None:
+                self._upload_manifest(manifest_type, question_diagnostics_path)
 
-                manifest.to_cloud(storage.path.join(question_diagnostics_path, f"{manifest_type}.json"))
-
-        # Upload the crash diagnostics events record.
-        storage_client.upload_from_string(
-            string=json.dumps(sent_messages or []),
-            cloud_path=storage.path.join(question_diagnostics_path, "messages.json"),
+        # Upload the messages received from any children before the crash.
+        self._storage_client.upload_from_string(
+            string=json.dumps(self.crash_diagnostics["questions"], cls=OctueJSONEncoder),
+            cloud_path=storage.path.join(question_diagnostics_path, "questions.json"),
         )
 
+    def _upload_values(self, values_type, question_diagnostics_path):
+        """Upload the values of the given type as part of the crash diagnostics.
 
-class AppFrom:
-    """A context manager that imports the module "app" from a file named "app.py" in the given directory on entry (by
-    making a temporary addition to the system path) and unloads it (by deleting it from `sys.modules`) on exit. It will
-    issue a warning if an existing module called "app" is already loaded. Usage example:
+        :param str values_type: one of "configuration_values" or "input_values"
+        :param str question_diagnostics_path: the path to a cloud directory to upload the values into
+        :return None:
+        """
+        self._storage_client.upload_from_string(
+            json.dumps(self.crash_diagnostics[values_type], cls=OctueJSONEncoder),
+            cloud_path=storage.path.join(question_diagnostics_path, f"{values_type}.json"),
+        )
 
-    ```python3
-    with AppFrom('/path/to/dir') as app:
-        Runner().run(app)
-    ```
+    def _upload_manifest(self, manifest_type, question_diagnostics_path):
+        """Upload the serialised manifest of the given type as part of the crash diagnostics.
 
-    :param str app_path: path to directory containing module named "app.py".
-    :return None:
-    """
+        :param str manifest_type: one of "configuration_manifest" or "input_manifest"
+        :param str question_diagnostics_path: the path to a cloud directory to upload the manifest into
+        :return None:
+        """
+        manifest = self.crash_diagnostics[manifest_type]
 
-    def __init__(self, app_path="."):
-        self.app_path = os.path.abspath(os.path.normpath(app_path))
-        logger.debug("Initialising AppFrom context at app_path %s", self.app_path)
-        self.app_module = None
-
-    def __enter__(self):
-        # Warn on an app present on the system path
-        if "app" in sys.modules.keys():
-            logger.warning(
-                "Module 'app' already on system path. Using 'AppFrom' context will yield unexpected results. Avoid "
-                "using 'app' as a python module, except for your main entrypoint."
+        # Upload each dataset and update its path in the manifest.
+        for dataset_name, dataset_path in manifest["datasets"].items():
+            new_dataset_path = storage.path.join(
+                question_diagnostics_path,
+                f"{manifest_type}_datasets",
+                dataset_name,
             )
 
-        # Insert the present directory first on the system path.
-        sys.path.insert(0, self.app_path)
+            Dataset(dataset_path).upload(new_dataset_path)
+            manifest["datasets"][dataset_name] = new_dataset_path
 
-        # Import the app from the present directory.
-        self.app_module = importlib.import_module("app")
-
-        # Immediately clean up the entry to the system path (don't use "remove" because if the user has it in their
-        # path, this'll be an unexpected side effect, and don't do it in cleanup in case the called code inserts a path)
-        sys.path.pop(0)
-        logger.debug("Imported app at app_path and cleaned up temporary modification to sys.path %s", self.app_path)
-        return self
-
-    def __exit__(self, exc_type, exc_value, traceback):
-        """Unload the imported module.
-
-        :return None:
-        """
-        try:
-            del sys.modules["app"]
-            logger.debug("Deleted app from sys.modules")
-
-        except KeyError:
-            context_manager_name = type(self).__name__
-            logger.warning(
-                f"Module 'app' was already removed from the system path prior to exiting the {context_manager_name} "
-                f"context manager. Using the {context_manager_name} context may yield unexpected results."
-            )
-
-
-class AnalysisLogHandlerSwitcher:
-    """A context manager that, when activated, takes the given logger, removes its handlers, and adds a local handler
-    and any other handlers provided to it. A formatter is applied to the handlers that includes the given analysis ID
-    in the logging context. On leaving the context, the logger's initial handlers are restored to it and any that were
-    added to it in the context are removed.
-
-    :param str analysis_id:
-    :param logger.Logger logger:
-    :param str analysis_log_level:
-    :param list(logging.Handler) extra_log_handlers:
-    :return None:
-    """
-
-    def __init__(self, analysis_id, logger, analysis_log_level, extra_log_handlers=None):
-        self.analysis_id = analysis_id
-        self.logger = logger
-        self.analysis_log_level = analysis_log_level
-        self.extra_log_handlers = extra_log_handlers or []
-        self.initial_handlers = []
-
-    def __enter__(self):
-        """Remove the initial handlers from the logger, create a formatter that includes the analysis ID, use the
-        formatter on the local and extra handlers, and add these handlers to the logger.
-
-        :return None:
-        """
-        self.initial_handlers = list(self.logger.handlers)
-        self._remove_log_handlers()
-
-        # Add the analysis ID to the logging metadata.
-        octue_formatter = create_octue_formatter(
-            get_log_record_attributes_for_environment(),
-            [f"analysis-{self.analysis_id}"],
+        # Upload manifest.
+        self._storage_client.upload_from_string(
+            json.dumps(manifest, cls=OctueJSONEncoder),
+            cloud_path=storage.path.join(question_diagnostics_path, f"{manifest_type}.json"),
         )
-
-        # Apply a local console `StreamHandler` to the logger.
-        apply_log_handler(formatter=octue_formatter, log_level=self.analysis_log_level)
-
-        if not self.extra_log_handlers:
-            return
-
-        uncoloured_octue_formatter = create_octue_formatter(
-            get_log_record_attributes_for_environment(),
-            [f"analysis-{self.analysis_id}"],
-            use_colour=False,
-        )
-
-        # Apply any other given handlers to the logger.
-        for extra_handler in self.extra_log_handlers:
-
-            # Apply an uncoloured log formatter to any file log handlers to keep them readable (i.e. to avoid the ANSI
-            # escape codes).
-            if type(extra_handler).__name__ == "FileHandler":
-                apply_log_handler(
-                    handler=extra_handler,
-                    log_level=self.analysis_log_level,
-                    formatter=uncoloured_octue_formatter,
-                )
-                continue
-
-            if (
-                type(extra_handler).__name__ == "MemoryHandler"
-                and getattr(extra_handler, "target")
-                and type(getattr(extra_handler, "target")).__name__ == "FileHandler"
-            ):
-                apply_log_handler(
-                    handler=extra_handler.target,
-                    log_level=self.analysis_log_level,
-                    formatter=uncoloured_octue_formatter,
-                )
-                continue
-
-            apply_log_handler(handler=extra_handler, log_level=self.analysis_log_level, formatter=octue_formatter)
-
-    def __exit__(self, *args):
-        """Remove the new handlers from the logger and re-add the initial handlers.
-
-        :return None:
-        """
-        self._remove_log_handlers()
-
-        for handler in self.initial_handlers:
-            self.logger.addHandler(handler)
-
-    def _remove_log_handlers(self):
-        """Remove all handlers from the logger.
-
-        :return None:
-        """
-        for handler in list(self.logger.handlers):
-            self.logger.removeHandler(handler)
